function bindArenaActionCreator(actionCreator, dispatch, sceneReducerKey) {
    return function () {
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        var action = actionCreator.apply(void 0, args);
        if (action && action._sceneReducerKey) {
            console.warn('"Action with redux-arena should not contain an user specified "_sceneReducerKey" property.\n' +
                ("Occurred in type: " + action.type + ", _sceneReducerKey: " + sceneReducerKey + "."));
        }
        typeof action === "object"
            ? dispatch(Object.assign({}, { _sceneReducerKey: sceneReducerKey }, action))
            : dispatch(action);
    };
}
export default function bindArenaActionCreators(actionCreators, dispatch, sceneReducerKey) {
    if (typeof actionCreators === "function" ||
        typeof actionCreators !== "object" ||
        actionCreators === null) {
        throw new Error("bindArenaActionCreators expected an object, instead received " + (actionCreators === null ? "null" : typeof actionCreators) + ". " +
            "Did you write \"import ActionCreators from\" instead of \"import * as ActionCreators from\"?");
    }
    var keys = Object.keys(actionCreators);
    var bindedActionCreators = {};
    for (var i = 0; i < keys.length; i++) {
        var key = keys[i];
        var actionCreator = actionCreators[key];
        if (typeof actionCreator === "function") {
            bindedActionCreators[key] = bindArenaActionCreator(actionCreator, dispatch, sceneReducerKey);
        }
    }
    return bindedActionCreators;
}
