import { applyMiddleware, compose } from "redux";
import ActionTypes from "../ActionTypes";
import createSagaMiddleware, { END } from "redux-saga";
import { getArenaInitState, arenaReducer } from "../reducers";
import createEnhancedStore from "./createEnhancedStore";
import rootSaga from "../sagas";
export default function createArenaStore(reducers, options) {
    if (reducers === void 0) { reducers = {}; }
    if (options === void 0) { options = {}; }
    var enhencers = options.enhencers, sagaOptions = options.sagaOptions, initialStates = options.initialStates, middlewares = options.middlewares;
    var sagaMiddleware = createSagaMiddleware(sagaOptions);
    var mergedMiddlewares = [sagaMiddleware];
    if (middlewares) {
        mergedMiddlewares = mergedMiddlewares.concat(middlewares);
    }
    var store = createEnhancedStore(Object.assign({
        arena: arenaReducer
    }, reducers), Object.assign({
        arena: getArenaInitState()
    }, initialStates || {}), enhencers
        ? compose.apply(void 0, [applyMiddleware.apply(void 0, mergedMiddlewares)].concat(enhencers)) : applyMiddleware.apply(void 0, mergedMiddlewares));
    sagaMiddleware.run(rootSaga, { store: store });
    store.close = function () { return store.dispatch(END); };
    store.runSaga = function (saga) {
        return store.dispatch({
            type: ActionTypes.ARENA_INIT_AUDIENCE_SAGA,
            saga: saga
        });
    };
    return store;
}
