import { Map, List } from "immutable";
import ActionTypes from "../ActionTypes";
import getArenaInitState from "./getArenaInitState";
function addStateTreeNode(state, pReducerKey, reducerKey) {
    var stateTree = state.stateTree, stateTreeDict = state.stateTreeDict;
    var newStateTree, newStateTreeDict;
    var plainNewNode = {
        pReducerKey: null,
        reducerKey: reducerKey,
        children: Map()
    };
    var newNode = Map(plainNewNode);
    if (pReducerKey == null) {
        newStateTree = stateTree.set(reducerKey, newNode);
        newStateTreeDict = stateTreeDict.set(reducerKey, Map({ path: List([reducerKey]) }));
    }
    else {
        var pPath = stateTreeDict.getIn([pReducerKey, "path"]);
        var pNode = stateTree.getIn(pPath);
        newNode = newNode.set("pReducerKey", pNode.get("reducerKey"));
        var path = pPath.concat(["children", reducerKey]);
        var newPNode = pNode.setIn(["children", reducerKey], newNode);
        newStateTree = stateTree.setIn(pPath, newPNode);
        newStateTreeDict = stateTreeDict.set(reducerKey, Map({ path: path, isObsolete: false }));
    }
    return Object.assign({}, state, {
        stateTree: newStateTree,
        stateTreeDict: newStateTreeDict
    });
}
function getReducerKeysOfNode(node, breakChecker) {
    if (breakChecker && breakChecker(node))
        return List();
    return node
        .get("children")
        .map(function (child) { return getReducerKeysOfNode(child, breakChecker); })
        .reduce(function (prev, cur) { return prev.concat(cur); }, List([node.get("reducerKey")]));
}
function disableStateTreeNode(state, reducerKey) {
    var stateTreeDict = state.stateTreeDict, stateTree = state.stateTree;
    var path = stateTreeDict.getIn([reducerKey, "path"]);
    var node = stateTree.getIn(path);
    var obsoleteKeyList = getReducerKeysOfNode(node, function (node) { return stateTreeDict.getIn([node.get("reducerKey"), "isObsolete"]) === true; });
    var deltaDict = Map(obsoleteKeyList.map(function (tmpKey) { return [
        tmpKey,
        stateTreeDict.get(tmpKey).set("isObsolete", true)
    ]; }));
    var newStateTreeDict = stateTreeDict.merge(deltaDict);
    return Object.assign({}, state, {
        stateTreeDict: newStateTreeDict
    });
}
function findNodeUpward(path, stateTree, stateTreeDict, checker) {
    var node = stateTree.getIn(path);
    var dictItem = stateTreeDict.get(node.get("reducerKey"));
    if (checker(node, dictItem)) {
        if (path.count() > 1) {
            var nextPath = findNodeUpward(path.skipLast(2).toList(), stateTree, stateTreeDict, checker);
            return nextPath == null ? path : nextPath;
        }
        else {
            return path;
        }
    }
    else {
        return null;
    }
}
function deleteStateTreeNode(state, reducerKey) {
    var stateTree = state.stateTree, stateTreeDict = state.stateTreeDict;
    var path = stateTreeDict.getIn([reducerKey, "path"]);
    var node = stateTree.getIn(path);
    if (node.get("children").count() === 0) {
        var path4Del = findNodeUpward(path, stateTree, stateTreeDict, function (pNode, dictItem) {
            return dictItem.get("isObsolete") === true &&
                pNode.get("children").count() === 1;
        });
        if (path4Del != null) {
            var obsoleteKeyList_1 = getReducerKeysOfNode(stateTree.getIn(path4Del));
            var newStateTree = stateTree.deleteIn(path4Del);
            var newStateTreeDict = stateTreeDict.filterNot(function (_, key) { return key === undefined || obsoleteKeyList_1.includes(key); });
            return Object.assign({}, state, {
                stateTree: newStateTree,
                stateTreeDict: newStateTreeDict
            });
        }
    }
    return state;
}
export default function reducer(state, action) {
    if (state === void 0) { state = getArenaInitState(); }
    switch (action.type) {
        case ActionTypes.ARENA_SET_STATE:
            return Object.assign({}, state, action.state);
        case ActionTypes.ARENA_REPLACE_STATE:
            return Object.assign({}, action.state);
        case ActionTypes.ARENA_GLOBAL_PROPSPICKER_LOCK:
            return Object.assign({ propsLock: action.lock });
        case ActionTypes.ARENA_STATETREE_NODE_ADD:
            return addStateTreeNode(state, action.pReducerKey, action.reducerKey);
        case ActionTypes.ARENA_STATETREE_NODE_DISABLE:
            return disableStateTreeNode(state, action.reducerKey);
        case ActionTypes.ARENA_STATETREE_NODE_DELETE:
            return deleteStateTreeNode(state, action.reducerKey);
        default:
            return state;
    }
}
