var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
import * as React from "react";
import * as PropTypes from "prop-types";
import ActionTypes from "../../core/ActionTypes";
import { createCurtainReducer } from "../../core/reducers";
import { addStateTreeNode, curtainAddReducer, buildCurtainReducerDict } from "../../utils";
import { curtainConnect } from "../BundleComponent";
function buildConnectedSceneBundle(reducerKey, store) {
    var sagaTaskPromise = new Promise(function (resolve) {
        return store.dispatch({
            type: ActionTypes.ARENA_CURTAIN_INIT_SAGA,
            reducerKey: reducerKey,
            setSagaTask: resolve
        });
    });
    return curtainConnect(reducerKey, function () {
        return store.dispatch({
            type: ActionTypes.ARENA_CURTAIN_CLEAR_REDUX,
            reducerKey: reducerKey,
            sagaTaskPromise: sagaTaskPromise
        });
    });
}
function getParentReducerKey(arenaReducerDict) {
    return (arenaReducerDict &&
        arenaReducerDict._arenaScene &&
        arenaReducerDict._arenaScene.reducerKey);
}
var ArenaScene = /** @class */ (function (_super) {
    __extends(ArenaScene, _super);
    function ArenaScene() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    ArenaScene.prototype.componentWillMount = function () {
        var _a = this.context, store = _a.store, arenaReducerDict = _a.arenaReducerDict;
        var _b = this.props, sceneBundle = _b.sceneBundle, sceneProps = _b.sceneProps, reducerKey = _b.reducerKey, vReducerKey = _b.vReducerKey;
        var newReducerKey = curtainAddReducer(store, reducerKey, createCurtainReducer);
        var parentReducerKey = getParentReducerKey(arenaReducerDict);
        addStateTreeNode(store, parentReducerKey, newReducerKey);
        var newArenaReducerDict = buildCurtainReducerDict(arenaReducerDict, newReducerKey, vReducerKey);
        var ConnectedBundleComponent = buildConnectedSceneBundle(newReducerKey, this.context.store);
        var connectedBundleElement = React.createElement(ConnectedBundleComponent, {
            arenaReducerDict: newArenaReducerDict,
            sceneBundle: sceneBundle,
            sceneProps: sceneProps
        });
        this.setState({
            parentReducerKey: parentReducerKey,
            arenaReducerDict: newArenaReducerDict,
            ConnectedBundleComponent: ConnectedBundleComponent,
            connectedBundleElement: connectedBundleElement
        });
    };
    ArenaScene.prototype.componentWillReceiveProps = function (nextProps, nextContext) {
        var refreshFlag = false;
        var state = Object.assign({}, this.state);
        var reducerKey = nextProps.reducerKey, vReducerKey = nextProps.vReducerKey, sceneBundle = nextProps.sceneBundle, sceneProps = nextProps.sceneProps;
        var curReducerKey = state.arenaReducerDict._arenaCurtain.reducerKey;
        var newReducerKey = curReducerKey;
        if (reducerKey != null && reducerKey !== curReducerKey) {
            refreshFlag = true;
            newReducerKey = curtainAddReducer(nextContext.store, reducerKey, createCurtainReducer);
            addStateTreeNode(nextContext.store, this.state.parentReducerKey, newReducerKey);
            state.ConnectedBundleComponent = buildConnectedSceneBundle(newReducerKey, nextContext.store);
        }
        if (nextContext.arenaReducerDict !== this.context.arenaReducerDict ||
            reducerKey !== this.props.reducerKey ||
            vReducerKey !== this.props.vReducerKey ||
            sceneBundle !== this.props.sceneBundle ||
            sceneProps !== this.props.sceneProps ||
            refreshFlag === true) {
            refreshFlag = true;
            state.arenaReducerDict = buildCurtainReducerDict(nextContext.arenaReducerDict, newReducerKey, nextProps.vReducerKey);
            state.connectedBundleElement = React.createElement(state.ConnectedBundleComponent, {
                sceneBundle: sceneBundle,
                sceneProps: sceneProps,
                arenaReducerDict: state.arenaReducerDict
            });
        }
        this.setState(state);
    };
    ArenaScene.prototype.render = function () {
        return this.state.connectedBundleElement;
    };
    ArenaScene.contextTypes = {
        store: PropTypes.any,
        arenaReducerDict: PropTypes.object
    };
    return ArenaScene;
}(React.Component));
export default ArenaScene;
